@@ -1,18 +1,23 @@
 require 'net/https'
 require 'cgi'
-require 'json'
 require 'uri'
 
+begin
+  require 'json'
+rescue LoadError
+  require File.join File.dirname(File.dirname(__FILE__)), 'vendor', 'json.rb'
+end
+
 # It just gists.
 module Gist
   extend self
 
-  VERSION = '4.1.1'
+  VERSION = '4.2.0'
 
   # A list of clipboard commands with copy and paste support.
   CLIPBOARD_COMMANDS = {
     'xclip'   => 'xclip -o',
-    'xsel'    => 'xsel -o',
+    'xsel -i'    => 'xsel -o',
     'pbcopy'  => 'pbpaste',
     'putclip' => 'getclip'
   }
@@ -131,6 +136,25 @@ module Gist
     end
   end
 
+  # Convert github url into raw file url
+  #
+  # Unfortunately the url returns from github's api is legacy,
+  # we have to taking a HTTPRedirection before appending it
+  # with '/raw'. Let's looking forward for github's api fix :)
+  #
+  # @param [String] url
+  # @return [String] the raw file url
+  def rawify(url)
+    uri = URI(url)
+    request = Net::HTTP::Get.new(uri.path)
+    response = http(uri, request)
+    if Net::HTTPSuccess === response
+      url + '/raw'
+    elsif Net::HTTPRedirection === response
+      rawify(response.header['location'])
+    end
+  end
+
   # Log the user into gist.
   #
   # This method asks the user for a username and password, and tries to obtain
@@ -143,44 +167,50 @@ module Gist
   # @see http://developer.github.com/v3/oauth/
   def login!(credentials={})
     puts "Obtaining OAuth2 access_token from github."
-    print "GitHub username: "
-    username = credentials[:username] || $stdin.gets.strip
-    print "GitHub password: "
-    password = credentials[:password] || begin
-      `stty -echo` rescue nil
-      $stdin.gets.strip
-    ensure
-      `stty echo` rescue nil
-    end
-    puts ""
-
-    request = Net::HTTP::Post.new("#{base_path}/authorizations")
-    request.body = JSON.dump({
-      :scopes => [:gist],
-      :note => "The gist gem",
-      :note_url => "https://github.com/ConradIrwin/gist"
-    })
-    request.content_type = 'application/json'
-    request.basic_auth(username, password)
-
-    response = http(api_url, request)
-
-    if Net::HTTPUnauthorized === response && response['X-GitHub-OTP']
-      print "2-factor auth code: "
-      twofa_code = $stdin.gets.strip
+    loop do
+      print "GitHub username: "
+      username = credentials[:username] || $stdin.gets.strip
+      print "GitHub password: "
+      password = credentials[:password] || begin
+        `stty -echo` rescue nil
+        $stdin.gets.strip
+      ensure
+        `stty echo` rescue nil
+      end
       puts ""
 
-      request['X-GitHub-OTP'] = twofa_code
-      response = http(api_url, request)
-    end
+      request = Net::HTTP::Post.new("#{base_path}/authorizations")
+      request.body = JSON.dump({
+        :scopes => [:gist],
+        :note => "The gist gem",
+        :note_url => "https://github.com/ConradIrwin/gist"
+      })
+      request.content_type = 'application/json'
+      request.basic_auth(username, password)
 
-    if Net::HTTPCreated === response
-      File.open(auth_token_file, 'w', 0600) do |f|
-        f.write JSON.parse(response.body)['token']
+      response = http(api_url, request)
+
+      if Net::HTTPUnauthorized === response && response['X-GitHub-OTP']
+        print "2-factor auth code: "
+        twofa_code = $stdin.gets.strip
+        puts ""
+
+        request['X-GitHub-OTP'] = twofa_code
+        response = http(api_url, request)
+      end
+
+      if Net::HTTPCreated === response
+        File.open(auth_token_file, 'w', 0600) do |f|
+          f.write JSON.parse(response.body)['token']
+        end
+        puts "Success! #{ENV[URL_ENV_NAME] || "https://github.com/"}settings/applications"
+        return
+      elsif Net::HTTPUnauthorized === response
+        puts "Error: #{JSON.parse(response.body)['message']}"
+        next
+      else
+        raise "Got #{response.class} from gist: #{response.body}"
       end
-      puts "Success! #{ENV[URL_ENV_NAME] || "https://github.com/"}settings/applications"
-    else
-      raise "Got #{response.class} from gist: #{response.body}"
     end
   rescue => e
     raise e.extend Error
@@ -235,8 +265,12 @@ module Gist
                %Q{<script src="#{json['html_url']}.js"></script>}
              when :html_url
                json['html_url']
+             when :raw_url
+               rawify(json['html_url'])
              when :short_url
                shorten(json['html_url'])
+             when :short_raw_url
+               shorten(rawify(json['html_url']))
              else
                json
              end
@@ -324,6 +358,7 @@ Could not find copy command, tried:
               elsif RUBY_PLATFORM =~ /linux/
                 %w(
                   sensible-browser
+                  xdg-open
                   firefox
                   firefox-bin
                 ).detect do |cmd|
